package com.sample.ikna.entity;

import io.jmix.core.entity.BaseUser;
import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.entity.annotation.SystemLevel;
import io.jmix.core.metamodel.annotation.DependsOnProperties;
import io.jmix.core.metamodel.annotation.InstanceName;
import org.springframework.security.core.GrantedAuthority;

import javax.persistence.*;
import javax.validation.constraints.Email;
import java.time.OffsetDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.UUID;

@DiscriminatorValue("U")
@Entity(name = "ikna_User")
@Table(name = "IKNA_USER", indexes = {
        @Index(name = "IDX_IKNA_USER_BY_USERNAME", columnList = "USERNAME", unique = true),
        @Index(name = "IDX_IKNA_USER_BY_EMAIL", columnList = "EMAIL"),
        @Index(name = "IDX_IKNA_USER_BY_TOKEN", columnList = "OAUTH_TOKEN")
})
@Inheritance(strategy = InheritanceType.JOINED)
@DiscriminatorColumn(name = "DTYPE", discriminatorType = DiscriminatorType.STRING, length = 1)
public class User implements BaseUser {

    @Id
    @Column(name = "ID", nullable = false)
    @JmixGeneratedValue
    private UUID id;

    @Version
    @Column(name = "VERSION", nullable = false)
    private Integer version;

    @Column(name = "USERNAME", nullable = false)
    protected String username;

    @SystemLevel
    @Column(name = "PASSWORD")
    protected String password;

    @Column(name = "FIRST_NAME")
    protected String firstName;

    @Column(name = "LAST_NAME")
    protected String lastName;

    @Email
    @Column(name = "EMAIL")
    protected String email;

    @Column(name = "ENABLED")
    protected Boolean enabled = true;

    @Column(name = "OAUTH_TOKEN")
    private String oauthToken;

    @Column(name = "NICK_NAME")
    private String nickName;

    @Column(name = "PRIVACY_POLICY_ACCEPTANCE_DATE")
    private OffsetDateTime privacyPolicyAcceptanceDate;

    @Column(name = "ACCEPTED_PRIVACY_POLICY_VERSION", length = 31)
    private String acceptedPrivacyPolicyVersion;

    @Column(name = "LAST_IP_ADDRESS")
    private String lastIpAddress;

    public String getLastIpAddress() {
        return lastIpAddress;
    }

    public void setLastIpAddress(String lastIpAddress) {
        this.lastIpAddress = lastIpAddress;
    }

    public String getAcceptedPrivacyPolicyVersion() {
        return acceptedPrivacyPolicyVersion;
    }

    public void setAcceptedPrivacyPolicyVersion(String acceptedPrivacyPolicyVersion) {
        this.acceptedPrivacyPolicyVersion = acceptedPrivacyPolicyVersion;
    }

    public OffsetDateTime getPrivacyPolicyAcceptanceDate() {
        return privacyPolicyAcceptanceDate;
    }

    public void setPrivacyPolicyAcceptanceDate(OffsetDateTime privacyPolicyAcceptanceDate) {
        this.privacyPolicyAcceptanceDate = privacyPolicyAcceptanceDate;
    }

    public String getNickName() {
        return nickName;
    }

    public void setNickName(String nickName) {
        this.nickName = nickName;
    }

    public String getOauthToken() {
        return oauthToken;
    }

    public void setOauthToken(String oauthToken) {
        this.oauthToken = oauthToken;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public String getPassword() {
        return password;
    }

    @Override
    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public Boolean getEnabled() {
        return enabled;
    }

    public void setEnabled(Boolean enabled) {
        this.enabled = enabled;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return Collections.emptyList();
    }

    @Override
    public boolean isAccountNonExpired() {
        return enabled;
    }

    @Override
    public boolean isAccountNonLocked() {
        return enabled;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return enabled;
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    @InstanceName
    @DependsOnProperties({"firstName", "lastName", "username"})
    @Override
    public String getDisplayName() {
        return String.format("%s %s [%s]", (firstName != null ? firstName : ""),
                (lastName != null ? lastName : ""), username);
    }
}