package com.sample.ikna.entity.billing;

import com.sample.ikna.entity.User;
import io.jmix.core.entity.annotation.EmbeddedParameters;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.Date;

@Table(name = "IKNA_PAYMENT_TRANSACTION", indexes = {
        @Index(name = "IDX_IKNA_PAYMENT_TRANSACTION_DATE", columnList = "DATE_"),
        @Index(name = "IDX_IKNA_PAYMENT_TRANSACTION_USER", columnList = "USER_ID")
})
@Entity(name = "ikna_PaymentTransaction")
public class PaymentTransaction extends FinancialOperation {
    private static final long serialVersionUID = -849967102707151840L;

    @LastModifiedBy
    @Column(name = "LAST_MODIFIED_BY")
    private String lastModifiedBy;

    @Temporal(TemporalType.DATE)
    @LastModifiedDate
    @Column(name = "LAST_MODIFIED_DATE")
    private Date lastModifiedDate;

    @CreatedBy
    @Column(name = "CREATED_BY")
    private String createdBy;

    @Temporal(TemporalType.DATE)
    @CreatedDate
    @Column(name = "CREATED_DATE")
    private Date createdDate;

    @Version
    @Column(name = "VERSION")
    private Integer version;

    @NotNull
    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "USER_ID", nullable = false)
    private User user;

    @Column(name = "STATE")
    private String state;

    @Embedded
    @EmbeddedParameters(nullAllowed = false)
    @AttributeOverrides({
            @AttributeOverride(name = "brand", column = @Column(name = "CARD_BRAND")),
            @AttributeOverride(name = "lastFourDigits", column = @Column(name = "CARD_LAST_4_DIGITS")),
            @AttributeOverride(name = "expiryDate", column = @Column(name = "CARD_EXPIRY_DATE")),
            @AttributeOverride(name = "holderName", column = @Column(name = "CARD_HOLDER_NAME")),
            @AttributeOverride(name = "paymentGateToken", column = @Column(name = "CARD_PAYMENT_GATE_TOKEN"))
    })
    private CardDetails cardDetails;

    @NotNull
    @Column(name = "REFERENCE_NUMBER", nullable = false)
    private String referenceNumber;

    public String getReferenceNumber() {
        return referenceNumber;
    }

    public void setReferenceNumber(String referenceNumber) {
        this.referenceNumber = referenceNumber;
    }

    public CardDetails getCardDetails() {
        return cardDetails;
    }

    public void setCardDetails(CardDetails cardDetails) {
        this.cardDetails = cardDetails;
    }

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public Integer getVersion() {
        return version;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Date getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate) {
        this.lastModifiedDate = lastModifiedDate;
    }

    public String getLastModifiedBy() {
        return lastModifiedBy;
    }

    public void setLastModifiedBy(String lastModifiedBy) {
        this.lastModifiedBy = lastModifiedBy;
    }

    public TransactionState getState() {
        return TransactionState.fromId(state);
    }

    public void setState(TransactionState state) {
        this.state = state != null ? state.getId() : null;
    }
}